=== modified file 'drizzled/table.cc'
--- drizzled/table.cc	2009-02-09 20:00:19 +0000
+++ drizzled/table.cc	2009-02-10 07:06:51 +0000
@@ -282,6 +282,248 @@
   return;
 }
 
+int parse_table_proto(Session *session, drizzle::Table &table, TABLE_SHARE *share)
+{
+  {
+    LEX_STRING engine_name= { (char*)table.engine().name().c_str(),
+			      strlen(table.engine().name().c_str()) };
+    share->db_plugin= ha_resolve_by_name(session, &engine_name);
+  }
+
+  // share->db_create_options FAIL
+  // share->db_options_in_use FAIL
+  share->mysql_version= DRIZZLE_VERSION_ID; // TODO: remove
+  share->null_field_first= 0;
+
+  drizzle::Table::TableOptions table_options;
+
+  if(table.has_options())
+    table_options= table.options();
+
+  share->avg_row_length= table_options.has_avg_row_length() ?
+    table_options.avg_row_length() : 0;
+
+  share->page_checksum= table_options.has_page_checksum() ?
+    (table_options.page_checksum()?HA_CHOICE_YES:HA_CHOICE_NO)
+    : HA_CHOICE_UNDEF;
+
+  share->row_type= table_options.has_row_type() ?
+    (enum row_type) table_options.row_type() : ROW_TYPE_DEFAULT;
+
+  share->block_size= table_options.has_block_size() ?
+    table_options.block_size() : 0;
+
+  share->table_charset= get_charset(table_options.has_collation_id()?
+				    table_options.collation_id() : 0);
+
+  if (!share->table_charset)
+  {
+    /* unknown charset in head[38] or pre-3.23 frm */
+    if (use_mb(default_charset_info))
+    {
+      /* Warn that we may be changing the size of character columns */
+      errmsg_printf(ERRMSG_LVL_WARN,
+		    _("'%s' had no or invalid character set, "
+		      "and default character set is multi-byte, "
+		      "so character column sizes may have changed"),
+		    share->path.str);
+    }
+    share->table_charset= default_charset_info;
+  }
+
+  share->null_field_first= 1;
+
+  share->db_record_offset= 1;
+
+  share->blob_ptr_size= portable_sizeof_char_ptr; // more bonghits.
+
+  share->db_low_byte_first= true;
+
+  share->max_rows= table_options.has_max_rows() ?
+    table_options.max_rows() : 0;
+
+  share->min_rows= table_options.has_min_rows() ?
+    table_options.min_rows() : 0;
+
+  share->keys= table.indexes_size();
+
+  share->key_parts= 0;
+  for(int indx= 0; indx < table.indexes_size(); indx++)
+    share->key_parts+= table.indexes(indx).index_part_size();
+
+  share->key_info= (KEY*) alloc_root(&share->mem_root,
+				     table.indexes_size() * sizeof(KEY)
+				     +share->key_parts*sizeof(KEY_PART_INFO));
+
+  KEY_PART_INFO *key_part;
+
+  key_part= reinterpret_cast<KEY_PART_INFO*>
+    (share->key_info+table.indexes_size());
+
+
+  ulong *rec_per_key= (ulong*) alloc_root(&share->mem_root,
+					    sizeof(ulong*)*share->key_parts);
+
+  share->keynames.count= table.indexes_size();
+  share->keynames.name= NULL;
+  share->keynames.type_names= (const char**)
+    alloc_root(&share->mem_root, sizeof(char*) * (table.indexes_size()+1));
+
+  share->keynames.type_lengths= (unsigned int*)
+    alloc_root(&share->mem_root,
+	       sizeof(unsigned int) * (table.indexes_size()+1));
+
+  share->keynames.type_names[share->keynames.count]= NULL;
+  share->keynames.type_lengths[share->keynames.count]= 0;
+
+  KEY* keyinfo= share->key_info;
+  for (int keynr=0; keynr < table.indexes_size(); keynr++, keyinfo++)
+  {
+    drizzle::Table::Index indx= table.indexes(keynr);
+
+    keyinfo->table= 0;
+    keyinfo->flags= 0;
+
+    if(indx.is_unique())
+      keyinfo->flags|= HA_NOSAME;
+
+    if(indx.has_options())
+    {
+      drizzle::Table::Index::IndexOptions indx_options= indx.options();
+      if(indx_options.pack_key())
+	keyinfo->flags|= HA_PACK_KEY;
+
+      if(indx_options.var_length_key())
+	keyinfo->flags|= HA_VAR_LENGTH_PART;
+
+      if(indx_options.null_part_key())
+	keyinfo->flags|= HA_NULL_PART_KEY;
+
+      if(indx_options.binary_pack_key())
+	keyinfo->flags|= HA_BINARY_PACK_KEY;
+
+      if(indx_options.has_partial_segments())
+	keyinfo->flags|= HA_KEY_HAS_PART_KEY_SEG;
+
+      if(indx_options.auto_generated_key())
+	keyinfo->flags|= HA_GENERATED_KEY;
+
+      if(indx_options.has_key_block_size())
+      {
+	keyinfo->flags|= HA_USES_BLOCK_SIZE;
+	keyinfo->block_size= indx_options.key_block_size();
+      }
+      else
+      {
+	keyinfo->block_size= 0;
+      }
+
+    }
+
+    switch(indx.type())
+    {
+    case drizzle::Table::Index::UNKNOWN_INDEX:
+      keyinfo->algorithm= HA_KEY_ALG_UNDEF;
+      break;
+    case drizzle::Table::Index::BTREE:
+      keyinfo->algorithm= HA_KEY_ALG_BTREE;
+      break;
+    case drizzle::Table::Index::RTREE:
+      keyinfo->algorithm= HA_KEY_ALG_RTREE;
+      break;
+    case drizzle::Table::Index::HASH:
+      keyinfo->algorithm= HA_KEY_ALG_HASH;
+      break;
+    case drizzle::Table::Index::FULLTEXT:
+      keyinfo->algorithm= HA_KEY_ALG_FULLTEXT;
+
+    default:
+      /* TODO: suitable warning ? */
+      keyinfo->algorithm= HA_KEY_ALG_UNDEF;
+      break;
+    }
+
+    keyinfo->key_length= indx.key_length();
+
+    keyinfo->key_parts= indx.index_part_size();
+
+    keyinfo->key_part= key_part;
+    keyinfo->rec_per_key= rec_per_key;
+
+    for(unsigned int partnr= 0;
+	partnr < keyinfo->key_parts;
+	partnr++, key_part++)
+    {
+      drizzle::Table::Index::IndexPart part;
+      part= indx.index_part(partnr);
+
+      *rec_per_key++=0;
+
+      key_part->field= NULL;
+      key_part->fieldnr= part.fieldnr();
+      key_part->null_bit= 0;
+      /* key_part->offset= ASS ASS ASS. Set later in the frm code */
+      /* key_part->null_offset is only set if null_bit (see later) */
+      /* key_part->key_type= */ /* I *THINK* this may be okay.... */
+      /* key_part->type ???? */
+      key_part->key_part_flag= 0;
+      if(part.has_in_reverse_order())
+	key_part->key_part_flag= part.in_reverse_order()? HA_REVERSE_SORT : 0;
+
+      key_part->length= part.compare_length();
+
+      key_part->store_length= key_part->length;
+    }
+
+    if(!indx.has_comment())
+    {
+      keyinfo->comment.length= 0;
+      keyinfo->comment.str= NULL;
+    }
+    else
+    {
+      keyinfo->flags|= HA_USES_COMMENT;
+      keyinfo->comment.length= indx.comment().length();
+      keyinfo->comment.str= strmake_root(&share->mem_root,
+					 indx.comment().c_str(),
+					 keyinfo->comment.length);
+    }
+
+    keyinfo->name= strmake_root(&share->mem_root,
+				indx.name().c_str(),
+				indx.name().length());
+
+    share->keynames.type_names[keynr]= keyinfo->name;
+    share->keynames.type_lengths[keynr]= indx.name().length();
+  }
+
+  share->keys_for_keyread.init(0);
+  share->keys_in_use.init(share->keys);
+
+  if(table_options.has_connect_string())
+  {
+    size_t len= table_options.connect_string().length();
+    const char* str= table_options.connect_string().c_str();
+
+    share->connect_string.length= len;
+    share->connect_string.str= strmake_root(&share->mem_root, str, len);
+  }
+
+  if(table_options.has_comment())
+  {
+    size_t len= table_options.comment().length();
+    const char* str= table_options.comment().c_str();
+
+    share->comment.length= len;
+    share->comment.str= strmake_root(&share->mem_root, str, len);
+  }
+
+  share->key_block_size= table_options.has_key_block_size() ?
+    table_options.key_block_size() : 0;
+
+  return 0;
+}
+
 /*
   Read table definition from a binary / text based .frm file
 
@@ -331,250 +573,27 @@
 
   drizzle::Table table;
 
-  if(drizzle_read_table_proto(proto_path.c_str(), &table)==0)
+  if((error= drizzle_read_table_proto(proto_path.c_str(), &table)))
   {
-    {
-      LEX_STRING engine_name= { (char*)table.engine().name().c_str(),
-				strlen(table.engine().name().c_str()) };
-      share->db_plugin= ha_resolve_by_name(session, &engine_name);
-    }
-
-    // share->db_create_options FAIL
-    // share->db_options_in_use FAIL
-    share->mysql_version= DRIZZLE_VERSION_ID; // TODO: remove
-    share->null_field_first= 0;
-
-    drizzle::Table::TableOptions table_options;
-    
-    if(table.has_options())
-      table_options= table.options();
-
-    share->avg_row_length= table_options.has_avg_row_length() ?
-      table_options.avg_row_length() : 0;
-
-    share->page_checksum= table_options.has_page_checksum() ?
-      (table_options.page_checksum()?HA_CHOICE_YES:HA_CHOICE_NO)
-      : HA_CHOICE_UNDEF;
-
-    share->row_type= table_options.has_row_type() ?
-      (enum row_type) table_options.row_type() : ROW_TYPE_DEFAULT;
-
-    share->block_size= table_options.has_block_size() ?
-      table_options.block_size() : 0;
-
-    share->table_charset= get_charset(table_options.has_collation_id()?
-				      table_options.collation_id() : 0);
-
-    if (!share->table_charset)
-    {
-      /* unknown charset in head[38] or pre-3.23 frm */
-      if (use_mb(default_charset_info))
-      {
-	/* Warn that we may be changing the size of character columns */
-	errmsg_printf(ERRMSG_LVL_WARN,
-		      _("'%s' had no or invalid character set, "
-			"and default character set is multi-byte, "
-			"so character column sizes may have changed"),
-		      share->path.str);
-      }
-      share->table_charset= default_charset_info;
-    }
-
-    share->null_field_first= 1;
-
-    share->db_record_offset= 1;
-
-    share->blob_ptr_size= portable_sizeof_char_ptr; // more bonghits.
-
-    share->db_low_byte_first= true;
-
-    share->max_rows= table_options.has_max_rows() ?
-      table_options.max_rows() : 0;
-
-    share->min_rows= table_options.has_min_rows() ?
-      table_options.min_rows() : 0;
-
-    share->keys= table.indexes_size();
-
-    share->key_parts= 0;
-    for(int indx= 0; indx < table.indexes_size(); indx++)
-      share->key_parts+= table.indexes(indx).index_part_size();
-
-    share->key_info= (KEY*) alloc_root(&share->mem_root,
-				       table.indexes_size() * sizeof(KEY)
-				       +share->key_parts*sizeof(KEY_PART_INFO));
-
-    KEY_PART_INFO *key_part;
-
-    key_part= reinterpret_cast<KEY_PART_INFO*>
-      (share->key_info+table.indexes_size());
-
-
-    ulong *rec_per_key= (ulong*) alloc_root(&share->mem_root,
-					    sizeof(ulong*)*share->key_parts);
-
-    share->keynames.count= table.indexes_size();
-    share->keynames.name= NULL;
-    share->keynames.type_names= (const char**)
-      alloc_root(&share->mem_root, sizeof(char*) * (table.indexes_size()+1));
-
-    share->keynames.type_lengths= (unsigned int*)
-      alloc_root(&share->mem_root,
-		 sizeof(unsigned int) * (table.indexes_size()+1));
-
-    share->keynames.type_names[share->keynames.count]= NULL;
-    share->keynames.type_lengths[share->keynames.count]= 0;
-
-    KEY* keyinfo= share->key_info;
-    for (int keynr=0; keynr < table.indexes_size(); keynr++, keyinfo++)
-    {
-      drizzle::Table::Index indx= table.indexes(keynr);
-
-      keyinfo->table= 0;
-      keyinfo->flags= 0;
-
-      if(indx.is_unique())
-	keyinfo->flags|= HA_NOSAME;
-
-      if(indx.has_options())
-      {
-	drizzle::Table::Index::IndexOptions indx_options= indx.options();
-	if(indx_options.pack_key())
-	  keyinfo->flags|= HA_PACK_KEY;
-
-	if(indx_options.var_length_key())
-	  keyinfo->flags|= HA_VAR_LENGTH_PART;
-
-	if(indx_options.null_part_key())
-	  keyinfo->flags|= HA_NULL_PART_KEY;
-
-	if(indx_options.binary_pack_key())
-	  keyinfo->flags|= HA_BINARY_PACK_KEY;
-
-	if(indx_options.has_partial_segments())
-	  keyinfo->flags|= HA_KEY_HAS_PART_KEY_SEG;
-
-	if(indx_options.auto_generated_key())
-	  keyinfo->flags|= HA_GENERATED_KEY;
-
-	if(indx_options.has_key_block_size())
-	{
-	  keyinfo->flags|= HA_USES_BLOCK_SIZE;
-	  keyinfo->block_size= indx_options.key_block_size();
-	}
-	else
-	{
-	  keyinfo->block_size= 0;
-	}
-
-      }
-
-      switch(indx.type())
-      {
-      case drizzle::Table::Index::UNKNOWN_INDEX:
-	keyinfo->algorithm= HA_KEY_ALG_UNDEF;
-	break;
-      case drizzle::Table::Index::BTREE:
-	keyinfo->algorithm= HA_KEY_ALG_BTREE;
-	break;
-      case drizzle::Table::Index::RTREE:
-	keyinfo->algorithm= HA_KEY_ALG_RTREE;
-	break;
-      case drizzle::Table::Index::HASH:
-	keyinfo->algorithm= HA_KEY_ALG_HASH;
-	break;
-      case drizzle::Table::Index::FULLTEXT:
-	keyinfo->algorithm= HA_KEY_ALG_FULLTEXT;
-
-      default:
-	/* TODO: suitable warning ? */
-	keyinfo->algorithm= HA_KEY_ALG_UNDEF;
-	break;
-      }
-      
-      keyinfo->key_length= indx.key_length();
-
-      keyinfo->key_parts= indx.index_part_size();
-
-      keyinfo->key_part= key_part;
-      keyinfo->rec_per_key= rec_per_key;
-
-      for(unsigned int partnr= 0;
-	  partnr < keyinfo->key_parts;
-	  partnr++, key_part++)
-      {
-	drizzle::Table::Index::IndexPart part;
-	part= indx.index_part(partnr);
-
-	*rec_per_key++=0;
-
-	key_part->field= NULL;
-	key_part->fieldnr= part.fieldnr();
-	key_part->null_bit= 0;
-	/* key_part->offset= ASS ASS ASS. Set later in the frm code */
-	/* key_part->null_offset is only set if null_bit (see later) */
-	/* key_part->key_type= */ /* I *THINK* this may be okay.... */
-	/* key_part->type ???? */
-	key_part->key_part_flag= 0;
-	if(part.has_in_reverse_order())
-	  key_part->key_part_flag= part.in_reverse_order()? HA_REVERSE_SORT : 0;
-
-	key_part->length= part.compare_length();
-
-	key_part->store_length= key_part->length;
-      }
-
-      if(!indx.has_comment())
-      {
-	keyinfo->comment.length= 0;
-	keyinfo->comment.str= NULL;
-      }
-      else
-      {
-	keyinfo->flags|= HA_USES_COMMENT;
-	keyinfo->comment.length= indx.comment().length();
-	keyinfo->comment.str= strmake_root(&share->mem_root,
-					   indx.comment().c_str(),
-					   keyinfo->comment.length);
-      }
-
-      keyinfo->name= strmake_root(&share->mem_root,
-				  indx.name().c_str(),
-				  indx.name().length());
-
-      share->keynames.type_names[keynr]= keyinfo->name;
-      share->keynames.type_lengths[keynr]= indx.name().length();
-    }
-
-    share->keys_for_keyread.init(0);
-    share->keys_in_use.init(share->keys);
-
-    if(table_options.has_connect_string())
-    {
-      size_t len= table_options.connect_string().length();
-      const char* str= table_options.connect_string().c_str();
-
-      share->connect_string.length= len;
-      share->connect_string.str= strmake_root(&share->mem_root, str, len);
-    }
-
-    if(table_options.has_comment())
-    {
-      size_t len= table_options.comment().length();
-      const char* str= table_options.comment().c_str();
-
-      share->comment.length= len;
-      share->comment.str= strmake_root(&share->mem_root, str, len);
-    }
-
-    share->key_block_size= table_options.has_key_block_size() ?
-      table_options.key_block_size() : 0;
-
-
-
-//    return 0;
+    if(error>0)
+    {
+      my_errno= error;
+      error= 1;
+    }
+    else
+    {
+      if(!table.IsInitialized())
+      {
+	error= 4;
+      }
+    }
+    goto err_not_open;
   }
 
+  parse_table_proto(session, table, share);
+
+  /* end of proto code... now to get some scraps from FRM */
+
   if ((file= open(path.c_str(), O_RDONLY)) < 0)
   {
     /*

=== modified file 'tests/r/varbinary.result'
--- tests/r/varbinary.result	2008-12-22 21:38:56 +0000
+++ tests/r/varbinary.result	2009-02-10 07:06:51 +0000
@@ -24,7 +24,7 @@
 1	2
 drop table t1;
 show create table t1;
-ERROR HY000: Incorrect information in file: './test/t1.frm'
+ERROR 42S02: Table 'test.t1' doesn't exist
 drop table t1;
 create table t1(a varbinary(255));
 insert into t1 values("aaa   ");

=== modified file 'tests/t/frm.test'
--- tests/t/frm.test	2009-01-08 23:37:52 +0000
+++ tests/t/frm.test	2009-02-10 07:06:51 +0000
@@ -9,6 +9,7 @@
 
 # Create a junk frm file on disk
 system echo "this is a junk file for test" >> $MYSQLTEST_VARDIR/master-data/test/t1.frm ;
+system echo "this is a junk file for test" >> $MYSQLTEST_VARDIR/master-data/test/t1.dfe ;
 --replace_column 6 # 7 # 8 # 9 #
 SHOW TABLE STATUS like 't1';
 --error 1033
@@ -16,5 +17,6 @@
 --error 1051
 drop table if exists t1;
 --remove_file $MYSQLTEST_VARDIR/master-data/test/t1.frm
+--remove_file $MYSQLTEST_VARDIR/master-data/test/t1.dfe
 
 #end bug 93

=== modified file 'tests/t/varbinary.test'
--- tests/t/varbinary.test	2008-12-22 21:38:56 +0000
+++ tests/t/varbinary.test	2009-02-10 07:06:51 +0000
@@ -53,7 +53,7 @@
 chmod 0777 $MYSQLTEST_VARDIR/master-data/test/t1.MYD;
 copy_file std_data/bug19371.MYI $MYSQLTEST_VARDIR/master-data/test/t1.MYI;
 chmod 0777 $MYSQLTEST_VARDIR/master-data/test/t1.MYI;
---error 1033
+--error ER_NO_SUCH_TABLE
 show create table t1;
 drop table t1;
 

